

There are 3 copy-and-paste implementations of __vmalloc() in
arch/{arm,sparc64,x86_64}/kernel/module.c.

I believe the only reason is that __vmalloc() doesn't allow
to specify parameters of __get_vm_area().

This patch splits __vmalloc() into 2 functions. The new one,
__vmalloc_area(), can be used as follows:

	vm_struct *area = __get_vm_area(...);
	void *addr = __vmalloc_area(area, gfp, prot);

Signed-off-by: Oleg Nesterov <oleg@tv-sign.ru>

Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/linux/vmalloc.h |    1 
 25-akpm/mm/vmalloc.c            |   52 ++++++++++++++++++++++------------------
 2 files changed, 30 insertions(+), 23 deletions(-)

diff -puN include/linux/vmalloc.h~vmalloc-introduce-__vmalloc_area-function include/linux/vmalloc.h
--- 25/include/linux/vmalloc.h~vmalloc-introduce-__vmalloc_area-function	2005-03-09 16:33:08.000000000 -0800
+++ 25-akpm/include/linux/vmalloc.h	2005-03-09 16:33:08.000000000 -0800
@@ -27,6 +27,7 @@ extern void *vmalloc(unsigned long size)
 extern void *vmalloc_exec(unsigned long size);
 extern void *vmalloc_32(unsigned long size);
 extern void *__vmalloc(unsigned long size, int gfp_mask, pgprot_t prot);
+extern void *__vmalloc_area(struct vm_struct *area, int gfp_mask, pgprot_t prot);
 extern void vfree(void *addr);
 
 extern void *vmap(struct page **pages, unsigned int count,
diff -puN mm/vmalloc.c~vmalloc-introduce-__vmalloc_area-function mm/vmalloc.c
--- 25/mm/vmalloc.c~vmalloc-introduce-__vmalloc_area-function	2005-03-09 16:33:08.000000000 -0800
+++ 25-akpm/mm/vmalloc.c	2005-03-09 16:33:08.000000000 -0800
@@ -389,32 +389,12 @@ void *vmap(struct page **pages, unsigned
 
 EXPORT_SYMBOL(vmap);
 
-/**
- *	__vmalloc  -  allocate virtually contiguous memory
- *
- *	@size:		allocation size
- *	@gfp_mask:	flags for the page level allocator
- *	@prot:		protection mask for the allocated pages
- *
- *	Allocate enough pages to cover @size from the page level
- *	allocator with @gfp_mask flags.  Map them into contiguous
- *	kernel virtual space, using a pagetable protection of @prot.
- */
-void *__vmalloc(unsigned long size, int gfp_mask, pgprot_t prot)
+void *__vmalloc_area(struct vm_struct *area, int gfp_mask, pgprot_t prot)
 {
-	struct vm_struct *area;
 	struct page **pages;
 	unsigned int nr_pages, array_size, i;
 
-	size = PAGE_ALIGN(size);
-	if (!size || (size >> PAGE_SHIFT) > num_physpages)
-		return NULL;
-
-	area = get_vm_area(size, VM_ALLOC);
-	if (!area)
-		return NULL;
-
-	nr_pages = size >> PAGE_SHIFT;
+	nr_pages = (area->size - PAGE_SIZE) >> PAGE_SHIFT;
 	array_size = (nr_pages * sizeof(struct page *));
 
 	area->nr_pages = nr_pages;
@@ -439,7 +419,7 @@ void *__vmalloc(unsigned long size, int 
 			goto fail;
 		}
 	}
-	
+
 	if (map_vm_area(area, prot, &pages))
 		goto fail;
 	return area->addr;
@@ -449,6 +429,32 @@ fail:
 	return NULL;
 }
 
+/**
+ *	__vmalloc  -  allocate virtually contiguous memory
+ *
+ *	@size:		allocation size
+ *	@gfp_mask:	flags for the page level allocator
+ *	@prot:		protection mask for the allocated pages
+ *
+ *	Allocate enough pages to cover @size from the page level
+ *	allocator with @gfp_mask flags.  Map them into contiguous
+ *	kernel virtual space, using a pagetable protection of @prot.
+ */
+void *__vmalloc(unsigned long size, int gfp_mask, pgprot_t prot)
+{
+	struct vm_struct *area;
+
+	size = PAGE_ALIGN(size);
+	if (!size || (size >> PAGE_SHIFT) > num_physpages)
+		return NULL;
+
+	area = get_vm_area(size, VM_ALLOC);
+	if (!area)
+		return NULL;
+
+	return __vmalloc_area(area, gfp_mask, prot);
+}
+
 EXPORT_SYMBOL(__vmalloc);
 
 /**
_
